=== modified file 'bzrlib/tests/blackbox/test_version_info.py'
--- bzrlib/tests/blackbox/test_version_info.py	2011-10-31 15:49:35 +0000
+++ bzrlib/tests/blackbox/test_version_info.py	2011-12-30 13:02:27 +0000
@@ -18,10 +18,10 @@
 
 import os
 
-from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.version_info_formats import VersionInfoBuilder
 
+
 class TestVersionInfo(TestCaseWithTransport):
 
     def test_invalid_format(self):
@@ -163,16 +163,16 @@
 
     def test_non_ascii(self):
         """Test that we can output non-ascii data"""
-        
+
         commit_message = u'Non-ascii message with character not in latin-1: \u1234'
-        
+
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a_file'])
         tree.add('a_file')
         tree.commit(commit_message)
         out, err = self.run_bzr(
             ['version-info', '--include-history'], encoding='latin-1')
-        
+
         self.assertContainsString(out, commit_message.encode('utf-8'))
 
     def test_revision(self):

=== modified file 'bzrlib/tests/test_version_info.py'
--- bzrlib/tests/test_version_info.py	2011-12-30 12:52:35 +0000
+++ bzrlib/tests/test_version_info.py	2011-12-30 13:02:27 +0000
@@ -222,6 +222,19 @@
         self.assertEqual(['r1', 'r4', 'unversioned', 'removed'],
                          file_rev_stanza.get_all('revision'))
 
+    def test_revision(self):
+        wt = self.create_branch()
+        self.build_tree(['branch/a', 'branch/c'])
+        wt.add('c')
+        wt.rename_one('b', 'd')
+
+        stanza = self.regen(wt, check_for_clean=True,
+            include_file_revisions=True, revision_id=wt.last_revision())
+        file_rev_stanza = self.get_one_stanza(stanza, 'file-revisions')
+        self.assertEqual(['', 'a', 'b'], file_rev_stanza.get_all('path'))
+        self.assertEqual(['r1', 'r3', 'r2'],
+                         file_rev_stanza.get_all('revision'))
+
 
 class PythonVersionInfoTests(VersionInfoTestCase):
 

=== modified file 'bzrlib/version_info_formats/__init__.py'
--- bzrlib/version_info_formats/__init__.py	2011-12-19 13:23:58 +0000
+++ bzrlib/version_info_formats/__init__.py	2011-12-30 13:02:27 +0000
@@ -70,7 +70,6 @@
         :param revision_id: Revision id to print version for (optional)
         """
         self._branch = branch
-        self._working_tree = working_tree
         self._check = check_for_clean
         self._include_history = include_revision_history
         self._include_file_revs = include_file_revisions
@@ -80,19 +79,27 @@
         self._file_revisions = {}
         self._revision_id = revision_id
 
+        if self._revision_id is None:
+            self._tree = working_tree
+            self._working_tree = working_tree
+        else:
+            self._tree = self._branch.repository.revision_tree(self._revision_id)
+            # the working tree is not relevant if an explicit revision was specified
+            self._working_tree = None
+
     def _extract_file_revisions(self):
         """Extract the working revisions for all files"""
 
         # Things seem clean if we never look :)
         self._clean = True
 
-        if self._working_tree is not None:
+        if self._working_tree is self._tree:
             basis_tree = self._working_tree.basis_tree()
             # TODO: jam 20070215 The working tree should actually be locked at
             #       a higher level, but this will do for now.
             self._working_tree.lock_read()
         else:
-            basis_tree = self._branch.basis_tree()
+            basis_tree = self._branch.repository.revision_tree(self._revision_id)
 
         basis_tree.lock_read()
         try:
@@ -100,7 +107,7 @@
             for info in basis_tree.list_files(include_root=True):
                 self._file_revisions[info[0]] = info[-1].revision
 
-            if not self._check or self._working_tree is None:
+            if not self._check or self._working_tree is not self._tree:
                 return
 
             delta = self._working_tree.changes_from(basis_tree,
@@ -140,11 +147,7 @@
     def _iter_revision_history(self):
         """Find the messages for all revisions in history."""
 
-        # Unfortunately, there is no WorkingTree.revision_history
-        if self._working_tree is not None:
-            last_rev = self._working_tree.last_revision()
-        else:
-            last_rev = self._branch.last_revision()
+        last_rev = self._get_revision_id()
 
         repository =  self._branch.repository
         repository.lock_read()

